' Ron Charlton explicitly places 'makeDiehardData64.vb' in the public domain.
' It may be used for any purpose.
'
' $Id: MakeDiehardData64.vb 1.7 2014-06-18 13:09:18-04 Ron Exp $ $Name: release1dot7 $
'

Option Strict On
Option Explicit On

Imports System
Imports System.IO

Module MakeDiehard_Data
	' This module makes a data file for testing with
	' George Marsaglia's Diehard software.
	' Author: Ron Charlton
	' Date:   2011-01-31
	' Revised: 2014-02-18 by Ron Charlton for 64 bits

	Public Sub MakeDiehardData64()

		' Diehard requires at least 80 million bits
		Const oneSequenceLengthInBits As Integer = 90000000
		Const oneSequenceLengthInBytes As Integer = oneSequenceLengthInBits \ 8
		Const numSequences As Integer = 1

		Dim sequence, j As Integer
		Dim ul As ULong
		Dim data(oneSequenceLengthInBytes - 1) As Byte	' Byte is unsigned

		Dim outFileName As String = "DiehardData64.out"
		Console.WriteLine("Writing MakeDiehardData64.vb output to file '{0}'", outFileName)

		' delete existing output file
		If My.Computer.FileSystem.FileExists(outFileName) Then
			My.Computer.FileSystem.DeleteFile(outFileName)
		End If

		' initialize Mersenne Twister with the default value
		Dim mt As New MTRandom64()

		' Append the sequences to the file one at a time.
		For sequence = 1 To numSequences

			' make a sequence of pseudorandom bits
			For j = 0 To oneSequenceLengthInBytes - 1

				' If (j Mod 8) = 0 Then get a pseudorandom number
				If (j And 7) = 0 Then ul = mt.genrand64_int64()

				' x86 (little-endian) byte order
				data(j) = CByte(ul And &HFFUL)
				ul >>= 8
			Next j

			' True means Append to file
			My.Computer.FileSystem.WriteAllBytes(outFileName, data, True)

		Next sequence

		' say "file was written"
		Console.WriteLine("Done.")
		Console.WriteLine("")
		Console.WriteLine("Press the <Enter> key to continue.")
		Console.ReadLine()

	End Sub
End Module
